<?php

namespace AdScale\Handlers;

use AdScale\Helpers\Helper;
use AdScale\Helpers\Logger;
use AdScale\Repository\AdscaleEntityChanges;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly


class WooHandlers {
	
	const ADSCALE_CHANGES_TYPE_CATEGORY = 'category';
	const ADSCALE_CHANGES_TYPE_PRODUCT = 'product';
	const ADSCALE_CHANGES_TYPE_ORDER = 'order';
	
	protected static $adscale_changes_cache = [];
	
	
	public static function change_add_to_cart_args( $args, $product ) {
		
		/* @var \WC_Product_Simple $product Product object */
		
		if ( $product->is_purchasable() && $product->is_in_stock() && $product->get_type() === 'simple' ) {
			$args['attributes']['data-adscale_product_value'] = $product->get_price();
		}
		
		return $args;
	}
	
	
	
	
	public static function trigger_after_add_to_cart_event( $cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data ) {
		
		/* @var \WC_Product_Simple $product Product object */
		
		$product       = wc_get_product( $product_id );
		$product_price = $product->get_price();
		
		Assets::add_to_cart_trigger_js( $product_id, $product_price );
	}
	
	
	
	
	/**
	 * Save adscale_ad_link on hook 'woocommerce_checkout_order_processed'
	 *
	 * @param int $order_id
	 * @param array $posted_data
	 * @param \WC_Abstract_Order $order
	 */
	public static function save_adscale_ad_link( $order_id, $posted_data, $order ) {
		$adscaleAdLink = Helper::getAdscaleAdLink();
		
		if ( $order_id && $adscaleAdLink ) {
			update_post_meta( $order_id, '_adscale_ad_link', $adscaleAdLink );
		}
	}
	
	
	/**
	 * Save adscale_cart_id on hook 'woocommerce_checkout_order_processed'
	 *
	 * @param int $order_id
	 * @param array $posted_data
	 * @param \WC_Abstract_Order $order
	 */
	public static function save_session_data_as_custom_order_meta_data( $order_id, $posted_data, $order ) {
		$wc_session_cookie_restored = Helper::getWCSessionCookieRestoredValue();
		if ( $order_id && is_string( $wc_session_cookie_restored ) ) {
			update_post_meta( $order_id, '_adscale_cart_id', $wc_session_cookie_restored );
		}
	}


    /**
     * Save adscale__optin on hook 'woocommerce_checkout_order_processed'
     *
     * @param int $order_id
     * @param array $posted_data
     * @param \WC_Abstract_Order $order
     */
    public static function save_optin_order( $order_id, $posted_data, $order ) {

        if ($order_id && Helper::getOptInEnabled()) {
            $wc_adscale__optin = isset($_POST['adscale__optin']) ? $_POST['adscale__optin'] : 0;
            $date = date(DATE_RFC3339);
            $order = wc_get_order($order_id);
            // this parameter is responsible for the checkbox on checkout
            update_post_meta($order_id, '_adscale__optin', $wc_adscale__optin);
            if ($wc_adscale__optin) {
                // if  is true / then this parameter saves time for the user in meta field key  _adscale__optin_date
                update_user_meta($order->get_user_id(), '_adscale__optin_date', $date);
            } else {
                // if  is false / then this parameter saves time for the user in meta field key  _adscale__optout_date
                update_user_meta($order->get_user_id(), '_adscale__optout_date', $date);
            }
        }
    }
	
	/**
	 * Handler for hooks 'woocommerce_order_status_{status}'
	 *
	 * @param int $order_id
	 * @param \WC_Abstract_Order $order
	 */
	public static function order_status_changed( $order_id, $order ) {
		if ( ! is_admin() || wp_doing_ajax() ) {
			$cookie_name  = \str_replace( '%ORDER_ID%', $order_id, 'adscale_order_%ORDER_ID%' );
			$cookie_value = Helper::get_order_value( $order_id );
			
			if ( in_array( $order->get_status(), [ 'on-hold', 'processing', 'completed' ], true ) ) {
				Helper::setCookie( $cookie_name, $cookie_value );
				// for endpoint adscale/LastOrder
				Helper::setCookie( 'adscale__last_order_id', $order_id );
			}
			
			if ( in_array( $order->get_status(), [ 'failed', 'cancelled' ], true ) ) {
				Helper::setCookie( $cookie_name, '', 1 );
				// for endpoint adscale/LastOrder
				Helper::setCookie( 'adscale__last_order_id', '', 1 );
			}
		}
	}
	
	
	
	/**
	 * Handler for hook 'woocommerce_after_order_object_save'
	 *
	 * @param \WC_Abstract_Order $order
	 */
	public static function new_order( $order ) {
		if ( ! is_admin() || wp_doing_ajax() ) {
			$order_id     = $order->get_id();
			$cookie_name  = \str_replace( '%ORDER_ID%', $order_id, 'adscale_order_%ORDER_ID%' );
			$cookie_value = Helper::get_order_value( $order_id );
			
			if ( 'pending' === $order->get_status() ) {
				Helper::setCookie( $cookie_name, $cookie_value );
			}
		}
	}
	
	
	
	public static function maybe_enable_wc_legacy_api() {
		$wc_legacy_api_auto_enable = Helper::getConfigSetting( 'shop/wc_legacy_api_auto_enable', false );
		
		if ( $wc_legacy_api_auto_enable ) {
			update_option( 'woocommerce_api_enabled', 'yes' );
		}
	}
	
	
	/**
	 * Add Adscale data to orders(Legacy API)
	 *
	 * @param $order_data
	 * @param $order
	 *
	 * @return array
	 */
	public static function wc_legacy_api_order_filter( $order_data, $order ) {
		$order_data['adscale_cart_id'] = get_post_meta( $order->get_id(), '_adscale_cart_id', true );
		
		return $order_data;
	}
	
	
	/**
	 * Add Adscale data to orders(WP Rest API)
	 *
	 * @param $response
	 * @param $object
	 * @param $request
	 *
	 * @return array
	 */
	public static function wc_api_order_filter( $response, $object, $request ) {
		$response->data['adscale_cart_id'] = get_post_meta( $object->get_id(), '_adscale_cart_id', true );
		
		return $response;
	}
	
	
	
	public static function wc_changes_process_product_meta( $post_id, $post ) {
		
		if ( empty( $post->post_type ) || ! in_array( $post->post_type, [ 'product', 'product_variation' ], true ) ) {
			return;
		}
		
		$product = wc_get_product( $post_id );
		self::process_changes_product( $product );
	}
	
	
	
	public static function wc_changes_new_and_update_product( $product_id ) {
		$product = wc_get_product( $product_id );
		self::process_changes_product( $product );
	}
	
	
	
	public static function wc_changes_trash( $post_id ) {
		self::wc_changes_trash_untrash( $post_id );
	}
	
	
	
	public static function wc_changes_untrash( $post_id ) {
		self::wc_changes_trash_untrash( $post_id );
	}
	
	
	
	protected static function wc_changes_trash_untrash( $post_id ) {
		$post = get_post( $post_id );
		
		if ( empty( $post->post_type ) ) {
			return;
		}
		
		if ( 'product' === $post->post_type ) {
			$product = wc_get_product( $post_id );
			self::process_changes_product( $product );
		}
	}
	
	
	public static function wc_changes_product_no_stock( $product ) {
		self::process_changes_product( $product );
	}
	
	
	
	
	protected static function process_changes_product( $product ) {
		if ( ! $product instanceof \WC_Product ) {
			return;
		}
		
		if ( $product instanceof \WC_Product_Variation ) {
			$changes_product_id = $product->get_parent_id();
		} else {
			$changes_product_id = $product->get_id();
		}
		
		if ( $changes_product_id ) {
			if ( ! self::adscaleChangesCheckExisting( self::ADSCALE_CHANGES_TYPE_PRODUCT, $changes_product_id ) ) {
				$success = AdscaleEntityChanges::add( self::ADSCALE_CHANGES_TYPE_PRODUCT, $changes_product_id );
			}
		} else {
			Logger::log( '[object_id: EMPTY]', self::ADSCALE_CHANGES_TYPE_PRODUCT . '>', 'ChangesHooks' );
		}
		
	}



    public static function wc_checkout_page_front(){
        $value=Helper::getOptInChecked()=='on'?1:0;

        echo '<div id="adscale__optin">';
        woocommerce_form_field( 'adscale__optin', array(
            'type'      => 'checkbox',
            'required'      => true,
            'class'     => array('input-checkbox'),
            'label'     => Helper::getOptInMessage(),
        ), $value );
        echo '</div>';
    }
	
	
	
	public static function wc_changes_update_order( $order_id ) {
		$order = wc_get_order( $order_id );
		self::process_changes_order( $order );
	}
	
	
	
	
	protected static function process_changes_order( $order ) {
		if ( ! $order instanceof \WC_Order ) {
			return;
		}
		
		$changes_order_id = $order->get_id();
		
		if ( $changes_order_id ) {
			if ( ! self::adscaleChangesCheckExisting( self::ADSCALE_CHANGES_TYPE_ORDER, $changes_order_id ) ) {
				$success = AdscaleEntityChanges::add( self::ADSCALE_CHANGES_TYPE_ORDER, $changes_order_id );
			}
		} else {
			Logger::log( '[object_id: EMPTY]', self::ADSCALE_CHANGES_TYPE_ORDER . '>', 'ChangesHooks' );
		}
	}
	
	
	
	
	public static function wc_changes_category( $term_id ) {
		self::process_changes_category( $term_id );
	}
	
	
	
	protected static function process_changes_category( $category_id ) {
		$changes_category_id = (int) $category_id;
		
		if ( $changes_category_id ) {
			if ( ! self::adscaleChangesCheckExisting( self::ADSCALE_CHANGES_TYPE_CATEGORY, $changes_category_id ) ) {
				$success = AdscaleEntityChanges::add( self::ADSCALE_CHANGES_TYPE_CATEGORY, $changes_category_id );
			}
		} else {
			Logger::log( '[object_id: EMPTY]', self::ADSCALE_CHANGES_TYPE_CATEGORY . '>', 'ChangesHooks' );
		}
	}
	
	
	
	
	protected static function adscaleChangesCheckExisting( $object_type, $object_id ) {
		$cache_key = "adscale_changes_{$object_type}_{$object_id}";
		
		$exists_cached = isset( self::$adscale_changes_cache[ $cache_key ] )
			? self::$adscale_changes_cache[ $cache_key ]
			: false;
		
		if ( true === $exists_cached ) {
			return true;
		}
		
		$count = AdscaleEntityChanges::countByTypeAndObjectId( $object_type, $object_id );
		
		$exists = is_numeric( $count ) && $count > 0;
		
		self::$adscale_changes_cache[ $cache_key ] = $exists;
		
		return $exists;
	}
}
