<?php

namespace AdScale\PluginApi;

use AdScale\Helpers\Helper;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly


class Orders extends PluginApiBase {
	
	
	protected static $defaultIds = [];
	
	
	public static function initContent() {
		global $wp;
		
		if ( isset( $_GET['adscaleOrders'] ) ) {
			$wp->query_vars['adscaleOrders'] = sanitize_key( wp_unslash( $_GET['adscaleOrders'] ) );
		}
		
		//  make sure endpoint request
		if ( ! isset( $wp->query_vars['adscaleOrders'] ) ) {
			return;
		}
		
		
		try {
			ob_start();
			
			// Error handler
			self::setErrorHandler();
			
			Helper::nocacheHeaders();
			
			// Check Request (Guards)
			self::checkRequest();
			
			ob_end_clean();
			
			// Handle Request
			self::handleRequest();
			
			// Default error
			Helper::sendResponseFormattedError( self::makeErrorBody() );
		} catch ( PluginApiException $ex ) {
			Helper::sendResponseFormattedError(
				self::makeErrorBody( $ex->getApiErrorCode(), $ex->getMessage(), $ex->getApiErrorDetails() )
			);
		} catch ( \Exception $ex ) {
			Helper::sendResponseFormattedError( self::makeErrorBody( $ex->getCode(), $ex->getMessage() ) );
		}
	}
	
	
	public static function checkRequest() {
		parent::checkRequest();
		self::initParams();
		
		// last id from the previous page
		if ( ! self::checkParamAfter() ) {
			throw new PluginApiException( 'Invalid argument value', 'I001', 'after' );
		}
		
		// Number of items per page, min
		if ( ! self::checkParamLimitForMin() ) {
			throw new PluginApiException( 'Invalid argument value', 'I001', 'limit' );
		}
		
		// Number of items per page, max
		if ( ! self::checkParamLimitForMax() ) {
			throw new PluginApiException( 'Invalid argument value', 'I001', 'limit' );
		}
		
		// Start date
		if ( self::checkIfRequestParamExists( 'start' ) && ! self::checkParamStartFormatValid() ) {
			throw new PluginApiException( 'Wrong format', 'F001', 'start, expected DDMMYYYY' );
		}
		
		// End date
		if ( self::checkIfRequestParamExists( 'end' ) && ! self::checkParamEndFormatValid() ) {
			throw new PluginApiException( 'Wrong format', 'F001', 'end, expected DDMMYYYY' );
		}
		
		// Changed After date
		if ( self::checkIfRequestParamExists( 'changed_after' ) && ! self::checkParamChangedAfterFormatValid() ) {
			throw new PluginApiException( 'Wrong format', 'F001', 'changed_after, expected DDMMYYYY' );
		}
	}
	
	
	public static function initParams() {
		self::$requestParams['after']         = Helper::getPostBodyDataValue( 'after' );
		self::$requestParams['limit']         = Helper::getPostBodyDataValue( 'limit' );
		self::$requestParams['start']         = Helper::getPostBodyDataValue( 'start' );
		self::$requestParams['end']           = Helper::getPostBodyDataValue( 'end' );
		self::$requestParams['ids']           = Helper::getPostBodyDataValue( 'ids' );
		self::$requestParams['changed_after'] = Helper::getPostBodyDataValue( 'changed_after' );
		
		// Set default params if not exists
		if ( null === self::$requestParams['after'] ) {
			self::$requestParams['after'] = 0;
		}
		if ( null === self::$requestParams['limit'] ) {
			self::$requestParams['limit'] = self::$defaultLimit;
		}
		if ( null === self::$requestParams['ids'] ) {
			self::$requestParams['ids'] = self::$defaultIds;
		}
		
		// Params Type Hinting
		self::$requestParams['after'] = (int) self::$requestParams['after'];
		self::$requestParams['limit'] = (int) self::$requestParams['limit'];
		self::$requestParams['ids']   = self::resolveIds( self::$requestParams['ids'] );
	}
	
	
	/**
	 * Last id from the previous page, check if is valid
	 *
	 * @return bool is valid
	 */
	public static function checkParamAfter() {
		return self::$requestParams['after'] >= 0;
	}
	
	
	/**
	 * Check min Number of items per page
	 *
	 * @return bool is valid
	 */
	public static function checkParamLimitForMin() {
		return self::$requestParams['limit'] > 0;
	}
	
	
	/**
	 * Check max Number of items per page
	 *
	 * @return bool is valid
	 */
	public static function checkParamLimitForMax() {
		return self::$requestParams['limit'] <= self::$maxLimit;
	}
	
	
	/**
	 * Check date param 'start'
	 *
	 * @return bool
	 */
	public static function checkParamStartFormatValid() {
		return ! empty( self::$requestParams['start'] ) && Helper::validateDate( self::$requestParams['start'], 'dmY' );
	}
	
	
	/**
	 * Check date param 'end'
	 *
	 * @return bool is valid
	 */
	public static function checkParamEndFormatValid() {
		return ! empty( self::$requestParams['end'] ) && Helper::validateDate( self::$requestParams['end'], 'dmY' );
	}
	
	
	/**
	 * Check date param 'changed_after'
	 *
	 * @return bool is valid
	 */
	public static function checkParamChangedAfterFormatValid() {
		return ! empty( self::$requestParams['changed_after'] ) && Helper::validateDate( self::$requestParams['changed_after'], 'dmY' );
	}
	
	
	/**
	 * Resolve Ids
	 *
	 * @param mixed $ids
	 *
	 * @return array
	 * @throws PluginApiException
	 */
	private static function resolveIds( $ids ) {
		if ( is_float( $ids ) ) {
			throw new PluginApiException( 'Invalid argument value', 'I001', 'ids' );
		}
		
		if ( is_numeric( $ids ) ) {
			$ids = [ (int) $ids ];
		}
		
		if ( ! is_array( $ids ) ) {
			throw new PluginApiException( 'Invalid argument value', 'I001', 'ids' );
		}
		
		$filteredIds = [];
		
		foreach ( $ids as $id ) {
			if ( ! is_numeric( $id ) || is_float( $id ) ) {
				throw new PluginApiException( 'Invalid argument value', 'I001', 'ids' );
			}
			$filteredIds[] = (int) $id;
		}
		
		return $filteredIds;
	}
	
	
	/**
	 * Handle API request
	 *
	 * @throws PluginApiException
	 */
	public static function handleRequest() {
		$orders_data = [];
		
		$args = [
			'orderby'  => 'ID',
			'order'    => 'ASC',
			'paginate' => false,
		];
		
		if ( self::$requestParams['limit'] ) {
			$args['limit'] = self::$requestParams['limit'];
		}
		
		$date_created_restrictions = false;
		if ( ! self::$requestParams['ids'] ) {
			$args = self::addDateCreatedRestrictions( $args );
			
			$date_created_restrictions = true;
		}
		
		if ( self::$requestParams['changed_after'] ) {
			$args = self::addDateModifiedRestrictions( $args );
			if ( ! $date_created_restrictions ) {
				$args = self::addDateCreatedRestrictions( $args );
			}
		}
		
		
		add_filter( 'posts_clauses', [ self::class, 'queryFilter' ], 777, 2 );
		$orders_objs = wc_get_orders( $args );
		remove_filter( 'posts_clauses', [ self::class, 'queryFilter' ], 777 );
		
		if ( ! is_array( $orders_objs ) ) {
			throw new PluginApiException( 'Unknown error', 'S000', 'Orders could not be found' );
		}
		
		foreach ( $orders_objs as $orders_obj ) {
			/* @var \WC_Order $orders_obj Order object */
			$order_data = self::getOrderData( $orders_obj );
			if ( ! $order_data ) {
				continue;
			}
			
			$orders_data[] = $order_data;
		}
		
		Helper::sendResponseFormattedSuccess( [
			'orders' => $orders_data,
		] );
	}
	
	
	public static function addDateCreatedRestrictions( $args ) {
		$start = self::$requestParams['start'] ? Helper::reformatDate( self::$requestParams['start'], 'dmY', 'Y-m-d' ) : '';
		$end   = self::$requestParams['end'] ? Helper::reformatDate( self::$requestParams['end'], 'dmY', 'Y-m-d' ) : '';
		
		if ( $start && ! $end ) {
			$args['date_created'] = ">={$start}";
		}
		if ( $end && ! $start ) {
			$args['date_created'] = "<={$end}";
		}
		if ( $start && $end ) {
			$args['date_created'] = "{$start}...{$end}";
		}
		
		return $args;
	}
	
	
	public static function addDateModifiedRestrictions( $args ) {
		$changed_after = self::$requestParams['changed_after'] ? Helper::reformatDate( self::$requestParams['changed_after'], 'dmY', 'Y-m-d' ) : '';
		
		if ( $changed_after ) {
			$args['date_modified'] = ">={$changed_after}";
		}
		
		return $args;
	}
	
	
	/**
	 * Filters all query clauses at once.
	 *
	 * @param string[] $clauses Associative array of the clauses for the query.
	 * @param \WP_Query $query The WP_Query instance (passed by reference).
	 *
	 * @return string[] $clauses Filtered Array of query SQL clauses.
	 */
	public static function queryFilter( $clauses, $query ) {
		global $wpdb;
		
		if ( self::$requestParams['ids'] ) {
			$post__in = implode( ',', array_map( 'absint', self::$requestParams['ids'] ) );
			
			$clauses['where'] .= " AND {$wpdb->posts}.ID IN ({$post__in})";
		}
		
		if ( self::$requestParams['after'] ) {
			$clauses['where'] .= ' AND ' . $wpdb->prepare( "{$wpdb->posts}.ID > %d", (int) self::$requestParams['after'] );
		}
		
		return $clauses;
	}
	
	
	/**
	 * Get Order data
	 *
	 * @param \WC_Order $order
	 *
	 * @return array
	 * @throws PluginApiException
	 */
	public static function getOrderData( $order ) {
		try {
			if ( ! $order instanceof \WC_Order ) {
				return [];
			}
			
			$order_id = (int) $order->get_id();
			
			// Custom adscale 'cart_id' (factually it is session cookie value)
			$cart_id = get_post_meta( $order_id, '_adscale_cart_id', true );
			$cart_id = $cart_id ? $cart_id : null;
			
			// date
			$date = $order->get_date_created();
			$date = $date instanceof \WC_DateTime ? $date->format( 'Y-m-d H:i:s' ) : null;

			// payment_date
			$payment_date = $order->get_date_paid();
			$payment_date = $payment_date instanceof \WC_DateTime ? $payment_date->format( 'Y-m-d H:i:s' ) : null;

			// update_date
			$update_date = $order->get_date_modified();
			$update_date = $update_date instanceof \WC_DateTime ? $update_date->format( 'Y-m-d H:i:s' ) : null;

			// customer info
			$customer_id = $order->get_customer_id();

            //  Opt-in Checkbox Setup
            $optin      = 'none';
            $optinValue = $order->get_meta( '_adscale__optin');

            if ($optinValue) {
                $optin = 'yes';
            } elseif ($optinValue === '0') {
                $optin = 'no';
            }

			if ( $customer_id ) {
				$customer = new \WC_Customer( $customer_id );

				$email = $customer->get_email(); // Get account email
				$phone = $customer->get_billing_phone();

				$first_name = $customer->get_first_name();
                $last_name = $customer->get_last_name();

			}
			
			$email = ! empty( $email ) ? $email : $order->get_billing_email();
			$phone = ! empty( $phone ) ? $phone : $order->get_billing_phone();

			$first_name = ! empty( $first_name ) ? $first_name : $order->get_billing_first_name();
			$last_name  = ! empty( $last_name ) ? $last_name : $order->get_billing_last_name();

			// country and state
			$shipping_country = $order->get_shipping_country();
			$shipping_state   = $order->get_shipping_state();
			$shipping_state   = Helper::getFullStateName( $shipping_country, $shipping_state );

			$billing_country = $order->get_billing_country();
			$billing_state   = $order->get_billing_state();
            $billing_state = Helper::getFullStateName($billing_country, $billing_state);

            // items
            $order_items_data = [];
            $order_items_objs = $order->get_items();
            foreach ($order_items_objs as $order_item_obj) {
                if ($order_item_obj instanceof \WC_Order_Item_Product) {
                    $order_items_data[] = self::getOrderItemData($order_item_obj);
                }
            }


            return [
                'id' => $order_id,
                'cart_id' => $cart_id,
                'date' => $date,
                'payment_date' => $payment_date,
                'update_date' => $update_date,
                'status' => $order->get_status(),
                'payment_method' => $order->get_payment_method_title(),
                'adscale_ad_link' => Helper::getSavedAdscaleAdLink($order_id),

                'currency' => $order->get_currency(),
				'total_price'     => Helper::formatPriceWithPrecision( $order->get_total() ),
				'shipping_price'  => Helper::formatPriceWithPrecision( self::getResolvedTotalShipping( $order ) ),
				'discount_amount' => Helper::formatPriceWithPrecision( self::getResolvedTotalDiscount( $order ) ),

				'ip'         => $order->get_customer_ip_address(),
				'email'      => $email,
				'phone'      => $phone,
				'first_name' => $first_name,
				'last_name'  => $last_name,
				'optin_sms'  => $optin,
				'optin_email'  => $optin,

				'shipping_country'  => $shipping_country,
				'shipping_state'    => $shipping_state,
				'shipping_city'     => $order->get_shipping_city(),
				'shipping_address'  => $order->get_shipping_address_1(),
				'shipping_address2' => $order->get_shipping_address_2(),
				'shipping_zip'      => $order->get_shipping_postcode(),

				'billing_country'  => $billing_country,
				'billing_state'    => $billing_state,
				'billing_city'     => $order->get_billing_city(),
				'billing_address'  => $order->get_billing_address_1(),
				'billing_address2' => $order->get_billing_address_2(),
				'billing_zip'      => $order->get_billing_postcode(),
				
				'items' => $order_items_data,
			];
		} catch ( \Exception $e ) {
			throw new PluginApiException( 'Unknown error', 'S000', $e->getMessage() );
		}
	}
	
	
	/**
	 * Get resolved total shipping
	 *
	 * @param \WC_Order $order
	 *
	 * @return string
	 */
	public static function getResolvedTotalShipping( $order ) {
		// Show shipping including tax.
		$shipping = $order->get_shipping_total() + $order->get_shipping_tax();
		
		return (string) $shipping;
	}
	
	
	
	
	/**
	 * Get resolved total discount
	 *
	 * @param \WC_Order $order
	 *
	 * @return string
	 */
	public static function getResolvedTotalDiscount( $order ) {
		return (string) $order->get_total_discount( true );
	}
	
	
	
	
	/**
	 * Get order item product data
	 *
	 * @param \WC_Order_Item_Product $order_item Order Item instance
	 *
	 * @return array
	 * @throws PluginApiException
	 */
	public static function getOrderItemData( $order_item ) {
		try {
			$quantity = $order_item->get_quantity();
			
			$total_price = (string) ( $order_item->get_total() + $order_item->get_total_tax() ); // Discounted total with tax
			
			$single_item_price = $quantity > 0 ? $total_price / $quantity : null;
			
			$single_item_price_excl_tax = $quantity > 0 ? $order_item->get_total() / $quantity : null;
			
			$total_discount = $order_item->get_subtotal() - $order_item->get_total();
			
			return [
				'product_id'                  => $order_item->get_product_id(),
				'variant_id'                  => $order_item->get_variation_id(),
				'quantity'                    => $order_item->get_quantity(),
				'total_price'                 => Helper::formatPriceWithPrecision( $total_price ),
				'single_item_price'           => Helper::formatPriceWithPrecision( $single_item_price ),
				'single_item_price_excl_tax'  => Helper::formatPriceWithPrecision( $single_item_price_excl_tax ),
				'single_item_discount_amount' => '0',
				'total_discount_amount'       => Helper::formatPriceWithPrecision( $total_discount ),
			];
		} catch ( \Exception $e ) {
			throw new PluginApiException( 'Unknown error', 'S000', $e->getMessage() );
		}
	}
}
