<?php

namespace AdScale\PluginApi;

use AdScale\Helpers\Helper;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly


class OrdersCount extends PluginApiBase {
	
	
	public static function initContent() {
		global $wp;
		
		if ( isset( $_GET['adscaleOrdersCount'] ) ) {
			$wp->query_vars['adscaleOrdersCount'] = sanitize_key( wp_unslash( $_GET['adscaleOrdersCount'] ) );
		}
		
		//  make sure endpoint request
		if ( ! isset( $wp->query_vars['adscaleOrdersCount'] ) ) {
			return;
		}
		
		
		try {
			ob_start();
			
			// Error handler
			self::setErrorHandler();
			
			Helper::nocacheHeaders();
			
			// Check Request (Guards)
			self::checkRequest();
			
			ob_end_clean();
			
			// Handle Request
			self::handleRequest();
			
			// Default error
			Helper::sendResponseFormattedError( self::makeErrorBody() );
		} catch ( PluginApiException $ex ) {
			Helper::sendResponseFormattedError(
				self::makeErrorBody( $ex->getApiErrorCode(), $ex->getMessage(), $ex->getApiErrorDetails() )
			);
		} catch ( \Exception $ex ) {
			Helper::sendResponseFormattedError( self::makeErrorBody( $ex->getCode(), $ex->getMessage() ) );
		}
	}
	
	
	public static function checkRequest() {
		parent::checkRequest();
		self::initParams();
		
		// Start date
		if ( self::checkIfRequestParamExists( 'start' ) && ! self::checkParamStartFormatValid() ) {
			throw new PluginApiException( 'Wrong format', 'F001', 'start, expected DDMMYYYY' );
		}
		
		// End date
		if ( self::checkIfRequestParamExists( 'end' ) && ! self::checkParamEndFormatValid() ) {
			throw new PluginApiException( 'Wrong format', 'F001', 'end, expected DDMMYYYY' );
		}
	}
	
	
	public static function initParams() {
		self::$requestParams['start'] = Helper::getPostBodyDataValue( 'start' );
		self::$requestParams['end']   = Helper::getPostBodyDataValue( 'end' );
	}
	
	
	/**
	 * Check date param 'start'
	 *
	 * @return bool
	 */
	public static function checkParamStartFormatValid() {
		return ! empty( self::$requestParams['start'] ) && Helper::validateDate( self::$requestParams['start'], 'dmY' );
	}
	
	
	/**
	 * Check date param 'end'
	 *
	 * @return bool is valid
	 */
	public static function checkParamEndFormatValid() {
		return ! empty( self::$requestParams['end'] ) && Helper::validateDate( self::$requestParams['end'], 'dmY' );
	}
	
	
	/**
	 * Handle API request
	 *
	 * @throws PluginApiException
	 */
	public static function handleRequest() {
		$args = [
			'limit'  => - 1,
			'return' => 'ids',
		];
		
		$start = self::$requestParams['start'] ? Helper::reformatDate( self::$requestParams['start'], 'dmY', 'Y-m-d' ) : '';
		$end   = self::$requestParams['end'] ? Helper::reformatDate( self::$requestParams['end'], 'dmY', 'Y-m-d' ) : '';
		
		if ( $start && ! $end ) {
			$args['date_created'] = ">={$start}";
		}
		if ( $end && ! $start ) {
			$args['date_created'] = "<={$end}";
		}
		if ( $start && $end ) {
			$args['date_created'] = "{$start}...{$end}";
		}
		
		$orders_ids = wc_get_orders( $args );
		
		if ( ! is_array( $orders_ids ) ) {
			throw new PluginApiException( 'Unknown error', 'S000', 'Orders count error' );
		}
		
		Helper::sendResponseFormattedSuccess( [ 'count' => count( $orders_ids ), ] );
	}
}
