<?php

namespace AdScale\PluginApi;

use AdScale\Helpers\Helper;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly


class Shipping extends PluginApiBase {
	
	
	protected static $defaultIds = [];
	
	
	public static function initContent() {
		global $wp;
		
		if ( isset( $_GET['adscaleShipping'] ) ) {
			$wp->query_vars['adscaleShipping'] = sanitize_key( wp_unslash( $_GET['adscaleShipping'] ) );
		}
		
		//  make sure endpoint request
		if ( ! isset( $wp->query_vars['adscaleShipping'] ) ) {
			return;
		}
		
		
		try {
			ob_start();
			
			// Error handler
			self::setErrorHandler();
			
			Helper::nocacheHeaders();
			
			// Check Request (Guards)
			self::checkRequest();
			
			ob_end_clean();
			
			// Handle Request
			self::handleRequest();
			
			// Default error
			Helper::sendResponseFormattedError( self::makeErrorBody() );
		} catch ( PluginApiException $ex ) {
			Helper::sendResponseFormattedError(
				self::makeErrorBody( $ex->getApiErrorCode(), $ex->getMessage(), $ex->getApiErrorDetails() )
			);
		} catch ( \Exception $ex ) {
			Helper::sendResponseFormattedError( self::makeErrorBody( $ex->getCode(), $ex->getMessage() ) );
		}
	}
	
	
	/**
	 * Handle API request
	 *
	 * @throws PluginApiException
	 */
	public static function handleRequest() {
		$zones_data = [];
		
		$zones = \WC_Shipping_Zones::get_zones();
		
		if ( ! is_array( $zones ) ) {
			throw new PluginApiException( 'Unknown error', 'S000', 'Shipping zones could not be found' );
		}
		
		foreach ( $zones as $zone ) {
			$zones_data[] = self::getShippingZoneData( $zone );
		}
		
		Helper::sendResponseFormattedSuccess( [
			'shipping_zones' => $zones_data,
		] );
	}
	
	
	
	/**
	 * Get Shipping Zone data
	 *
	 * @param array $sz_data
	 *
	 * @return array Resolved Shipping Zone data
	 * @throws PluginApiException
	 */
	public static function getShippingZoneData( $sz_data ) {
		try {
			return [
				'id'               => Helper::getArrayValue( $sz_data, 'zone_id', null, 'intval' ),
				'name'             => Helper::getArrayValue( $sz_data, 'zone_name', null ),
				'locations'        => self::getShippingZoneLocationsData( $sz_data ),
				'shipping_methods' => self::getShippingZoneMethodsData( $sz_data ),
			];
		} catch ( \Exception $e ) {
			throw new PluginApiException( 'Unknown error', 'S000', $e->getMessage() );
		}
	}
	
	
	
	/**
	 * Get Shipping Zone Locations data
	 *
	 * @param array $sz_data
	 *
	 * @return array Resolved data
	 */
	public static function getShippingZoneLocationsData( $sz_data ) {
		$zone_locations = Helper::getArrayValue( $sz_data, 'zone_locations', [] );
		
		$locations = [];
		foreach ( $zone_locations as $zone_location ) {
			$type = Helper::getObjProperty( $zone_location, 'type' );
			$code = Helper::getObjProperty( $zone_location, 'code' );
			
			if ( 'continent' === $type ) {
				$name = Helper::getFullContinentName( $code );
			} elseif ( 'country' === $type ) {
				$name = Helper::getFullCountryName( $code );
			} elseif ( 'state' === $type ) {
				$state_data         = explode( ':', $code );
				$state_country_code = Helper::getArrayValue( $state_data, 0, null );
				$state_code         = Helper::getArrayValue( $state_data, 1, null );
				$state_str_p1       = Helper::getFullCountryName( $state_country_code );
				$state_str_p2       = Helper::getFullStateName( $state_country_code, $state_code );
				$name               = "$state_str_p1, $state_str_p2";
			} elseif ( 'postcode' === $type ) {
				$name = '';
			} else {
				$name = '';
			}
			
			$locations[] = [
				'type' => $type,
				'code' => $code,
				'name' => $name,
			];
		}
		
		return $locations;
	}
	
	
	
	/**
	 * Get Shipping Zone Methods data
	 *
	 * @param array $sz_data
	 *
	 * @return array Resolved data
	 */
	public static function getShippingZoneMethodsData( $sz_data ) {
		$shipping_methods = Helper::getArrayValue( $sz_data, 'shipping_methods', [] );
		
		$shipping_methods_data = [];
		foreach ( $shipping_methods as $shipping_method ) {
			if ( $shipping_method instanceof \WC_Shipping_Method && $shipping_method->is_enabled() ) {
				$shipping_methods_data[] = [
					'id'          => $shipping_method->get_instance_id(),
					'name'        => $shipping_method->get_title(),
					'method_name' => $shipping_method->get_method_title(),
				];
			}
		}
		
		return $shipping_methods_data;
	}
}
