<?php

namespace AdScale\ServiceApi;

use AdScale\Helpers\Helper;
use AdScale\Helpers\Logger;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly


abstract class ServiceApiBase {
	
	protected static $requestParams = [];
	
	
	
	public static function handleRequest() {
		Helper::sendResponseFormattedSuccess( null );
	}
	
	
	
	/**
	 * The set of checks
	 *
	 * @throws ServiceApiException
	 */
	public static function checkRequest() {
		// Wrong HTTP Method guard
		if ( ! self::checkHTTPMethod() ) {
			throw new ServiceApiException( 'Unsupported method', 'S001', Helper::getCurrentHttpMethod() );
		}
		
		// Missing Key guard
		if ( ! self::checkKeyExists() ) {
			throw new ServiceApiException( 'Missing argument', 'M001', 'key' );
		}
		
		// Key not valid guard
		if ( ! self::checkKeyValid() ) {
			throw new ServiceApiException( 'Key validation failed', 'A001' );
		}
	}
	
	
	
	
	/**
	 * Checking if param passed
	 *
	 * @param $paramName
	 *
	 * @return bool is exist
	 */
	public static function checkIfRequestParamExists( $paramName ) {
		return null !== self::$requestParams[ $paramName ];
	}
	
	
	
	
	/**
	 * Return true if Request HTTP method as needed, else - false
	 *
	 * @return bool
	 */
	public static function checkHTTPMethod() {
		return 'POST' === Helper::getCurrentHttpMethod();
	}
	
	
	
	
	/**
	 * Return true if Key Exists, else - false
	 *
	 * @return bool
	 */
	public static function checkKeyExists() {
		return null !== Helper::getPostBodyDataValue( 'key' );
	}
	
	
	
	
	/**
	 * Return true if Key Valid, else - false
	 *
	 * @return bool
	 */
	public static function checkKeyValid() {
		$key = Helper::getAccessKey();
		
		return ! empty( $key ) && Helper::validateSecKey( $key );
	}
	
	
	
	
	/**
	 * Making error body for response
	 *
	 * @param string|int $code
	 * @param string $message
	 * @param string $details
	 *
	 * @return array
	 */
	public static function makeErrorBody( $code = 'S000', $message = 'Unknown error', $details = '' ) {
		$code = ( ! is_int( $code ) && ! is_string( $code ) ) ? 'S000' : $code;
		Logger::log( "code: {$code}, message: {$message}, details: {$details}", 'Error > ', 'ServiceApi_' . self::getCalledClassShortName() );
		
		return [  // json format : {error: {code: "%code%, message: '%message%'}}
			'error' => [
				'code'    => (string) $code,
				'message' => $message,
				'details' => $details,
			],
		];
	}
	
	
	
	
	public static function setErrorHandler() {
		// for disabling wordpress default shutdown handler for fatal errors
		if ( ! defined( 'WP_SANDBOX_SCRAPING' ) ) {
			define( 'WP_SANDBOX_SCRAPING', true );
		}
		
		ini_set( 'html_errors', 'off' );
		set_error_handler( [ static::class, 'errorHandler' ] );
		register_shutdown_function( [ static::class, 'errorShutdownHandler' ] );
	}
	
	
	
	
	/**
	 * Used to replace the default PHP error handler.
	 *
	 * @param int $errno contains the level of the error
	 * @param string $errstr contains the error message
	 * @param string $errfile errfile, which contains the filename that the error was raised in
	 * @param int $errline errline, which contains the line number the error was raised at
	 *
	 * @return bool Always return true to avoid the default PHP error handler
	 */
	public static function errorHandler( $errno, $errstr, $errfile, $errline ) {
		/*
		$_errno = $errno & error_reporting();
		if ( $_errno === 0 ) {
			return true;
		} */
		
		$errorType = [
			E_ERROR             => 'Error', // handle at shutdown
			E_WARNING           => 'Warning',
			E_PARSE             => 'Parse', // handle at shutdown
			E_NOTICE            => 'Notice',
			E_CORE_ERROR        => 'Core Error', // handle at shutdown
			E_CORE_WARNING      => 'Core Warning', // handle at shutdown
			E_COMPILE_ERROR     => 'Compile Error', // handle at shutdown
			E_COMPILE_WARNING   => 'Compile Warning', // handle at shutdown
			E_USER_ERROR        => 'Error',
			E_USER_WARNING      => 'User warning',
			E_USER_NOTICE       => 'User notice',
			E_STRICT            => 'Runtime Notice', // handle at shutdown
			E_RECOVERABLE_ERROR => 'Recoverable error',
			E_DEPRECATED        => 'Deprecated notice',
			E_USER_DEPRECATED   => 'Deprecated notice',
		];
		
		$type = isset( $errorType[ $errno ] ) ? $errorType[ $errno ] : 'Unknown error';
		
		$errorMessage = "[PHP $type level#{$errno}] $errstr ($errfile, line $errline)";
		
		$trace = '';
		
		if ( function_exists( 'debug_backtrace' ) ) {
			$backtrace = debug_backtrace();
			if ( $backtrace ) {
				$trace = ' Trace: ';
			}
			$backtrace = array_reverse( $backtrace );
			array_shift( $backtrace );
			foreach ( $backtrace as $i => $l ) {
				$class    = ! empty( $l['class'] ) ? $l['class'] : '';
				$type     = ! empty( $l['type'] ) ? $l['type'] : '';
				$function = ! empty( $l['function'] ) ? $l['function'] : '';
				$file     = ! empty( $l['file'] ) ? $l['file'] : '';
				$line     = ! empty( $l['line'] ) ? $l['line'] : '';
				
				if ( ! empty( $class ) || ! empty( $type ) || ! empty( $function ) ) {
					$trace .= "[$i] in function {$class}{$type}{$function}";
				}
				if ( ! empty( $file ) ) {
					$trace .= " in {$file}";
				}
				if ( ! empty( $line ) ) {
					$trace .= " on line {$line}";
				}
				$trace .= ', ';
			}
			$trace = rtrim( $trace, ', ' );
		}
		
		$errorMessage .= $trace;
		
		//$ex = new \Exception;
		//$errorMessage .= ' Trace: '. $ex->getTraceAsString();
		
		switch ( $errno ) {
			case E_DEPRECATED:
			case E_USER_DEPRECATED:
				break;
			case E_NOTICE:
			case E_USER_NOTICE:
			case E_STRICT:
				Logger::log( $errorMessage, 'PHP Notice > ', 'ServiceApi_' . self::getCalledClassShortName(), 0, 1 );
				break;
			case E_WARNING:
			case E_USER_WARNING:
			case E_RECOVERABLE_ERROR:
				Logger::log( $errorMessage, 'PHP Warning > ', 'ServiceApi_' . self::getCalledClassShortName(), 0, 1 );
				break;
			case E_ERROR:
			case E_PARSE:
			case E_CORE_ERROR:
			case E_CORE_WARNING:
			case E_COMPILE_ERROR:
			case E_COMPILE_WARNING:
			case E_USER_ERROR:
			default:
				Logger::log( $errorMessage, 'PHP Error > ', 'ServiceApi_' . self::getCalledClassShortName(), 0, 1 );
				Helper::sendResponseFormattedError(
					static::makeErrorBody( 'S000', 'Unknown error', $errorMessage )
				);
		}
		
		return true;
	}
	
	
	
	
	public static function errorShutdownHandler() {
		$error_types_to_handle = [
			E_ERROR,
			E_PARSE,
			E_CORE_ERROR,
			E_CORE_WARNING,
			E_COMPILE_ERROR,
			E_COMPILE_WARNING,
			E_USER_ERROR,
			E_STRICT,
		];
		
		$last_err = error_get_last();
		
		if ( isset( $last_err['type'] ) && in_array( $last_err['type'], $error_types_to_handle, true ) ) {
			static::errorHandler(
				$last_err['type'],
				$last_err['message'],
				isset( $last_err['file'] ) ? $last_err['file'] : '',
				isset( $last_err['line'] ) ? $last_err['line'] : ''
			);
		}
	}
	
	
	
	
	/**
	 * Getting called class name without namespace
	 *
	 * @return string
	 */
	public static function getCalledClassShortName() {
		try {
			$reflectionClass = new \ReflectionClass( get_called_class() );
			
			return $reflectionClass->getShortName();
		} catch ( \Exception $ex ) {
			return '';
		}
	}
}
